@@ -197,7 +197,7 @@ struct check_result *check_no_self(__attribute__((unused)) const struct check_da
 	if (node->flavor != NODE_AV_RULE && node->flavor != NODE_XAV_RULE) {
 		return alloc_internal_error("Bad node type given to check C-007");
 	}
-	struct av_rule_data *av_data = node->data.av_data;
+	const struct av_rule_data *av_data = node->data.av_data;
 
 	if (av_data->sources->next ||
 	    av_data->targets->next ||
@@ -512,24 +512,32 @@ struct check_result *check_no_explicit_declaration(const struct check_data *data
 		return NULL;
 	}
 
-	struct string_list *names = get_names_in_node(node);
+	struct name_list *names = get_names_in_node(node);
 
-	const struct string_list *name = names;
+	const struct name_list *name = names;
 	/* In declarations skip the first name, which is the new declared type */
 	if (node->flavor == NODE_DECL) {
 		name = name->next;
 	}
 
 	for (; name; name = name->next) {
+		const struct name_data *ndata = name->data;
 		const char *mod_name;
 		enum decl_flavor flavor;
 
-		if ((mod_name = look_up_in_decl_map(name->string, DECL_TYPE))) {
+		if (name_is_type(ndata) && (mod_name = look_up_in_decl_map(ndata->name, DECL_TYPE))) {
 			flavor = DECL_TYPE;
-		} else if ((mod_name = look_up_in_decl_map(name->string, DECL_ATTRIBUTE))) {
+		} else if (name_is_typeattr(ndata) && (mod_name = look_up_in_decl_map(ndata->name, DECL_ATTRIBUTE))) {
 			flavor = DECL_ATTRIBUTE;
-		} else if ((mod_name = look_up_in_decl_map(name->string, DECL_ATTRIBUTE_ROLE))) {
+		} else if (name_is_roleattr(ndata) && (mod_name = look_up_in_decl_map(ndata->name, DECL_ATTRIBUTE_ROLE))) {
 			flavor = DECL_ATTRIBUTE_ROLE;
+		} else if (name_is_class(ndata) && look_up_in_decl_map(name->data->name, DECL_CLASS)) {
+			// Ignore kernel classes
+			if (!userspace_class_support || !is_userspace_class(name->data->name, name->data->traits)) {
+				continue;
+			}
+			flavor = DECL_CLASS;
+			mod_name = NULL;
 		// Do not check for roles: in refpolicy they are defined in the kernel module
 		// and used in role modules (like unprivuser)
 		} else {
@@ -537,21 +545,28 @@ struct check_result *check_no_explicit_declaration(const struct check_data *data
 			continue;
 		}
 
-		if (0 != strcmp(data->mod_name, mod_name)) {
+		if (!mod_name || 0 != strcmp(data->mod_name, mod_name)) {
 			// It may be required
-			if (!has_require(node, name->string, flavor)) {
+			if (!has_require(node, ndata->name, flavor)) {
 				// We didn't find a require block with this name
-				struct check_result *to_ret = make_check_result('W', W_ID_NO_EXPLICIT_DECL,
-										"No explicit declaration for %s from module %s.  You should access it via interface call or use a require block.",
-										name->string, mod_name);
-				free_string_list(names);
+				struct check_result *to_ret;
+				if (flavor == DECL_CLASS) {
+					to_ret = make_check_result('W', W_ID_NO_EXPLICIT_DECL,
+					                           "No explicit declaration for userspace class %s.  You should access it via interface call or use a require block.",
+					                           ndata->name);
+				} else {
+					to_ret = make_check_result('W', W_ID_NO_EXPLICIT_DECL,
+					                           "No explicit declaration for %s from module %s.  You should access it via interface call or use a require block.",
+					                           ndata->name, mod_name);
+				}
+				free_name_list(names);
 				return to_ret;
 			}
 			// Otherwise, keep checking other names in this node
 		}
 	}
 
-	free_string_list(names);
+	free_name_list(names);
 	return NULL;
 }
 
